﻿namespace U2UC.Metro.Controls
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Media;
    using Windows.UI.Xaml.Shapes;

    [TemplatePart(Name = NeedlePartName, Type = typeof(Path))]
    public sealed class RadialGaugeControl : Control, INotifyPropertyChanged
    {
        private const string NeedlePartName = "PART_Needle";

        #region Dependency Properties

        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", typeof(double), typeof(RadialGaugeControl), new PropertyMetadata(0.0));

        public double Minimum
        {
            get { return (double)GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", typeof(double), typeof(RadialGaugeControl), new PropertyMetadata(100.0));

        public double Maximum
        {
            get { return (double)GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof(double), typeof(RadialGaugeControl), new PropertyMetadata(0.0, OnValueChanged));

        private static void OnValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            RadialGaugeControl c = (RadialGaugeControl)d;
            c.ValueAngle = c.ValueToAngle(c.Value);
            if (c.Needle != null)
            {
                c.Needle.RenderTransform = new RotateTransform() { Angle = c.ValueAngle };
            }
            c.OnPropertyChanged("ValueAngle");
        }

        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        public static readonly DependencyProperty UnitProperty =
            DependencyProperty.Register("Unit", typeof(string), typeof(RadialGaugeControl), new PropertyMetadata(string.Empty));

        public string Unit
        {
            get { return (string)GetValue(UnitProperty); }
            set { SetValue(UnitProperty, value); }
        }

        public static readonly DependencyProperty QualitativeRangesProperty =
            DependencyProperty.Register("QualitativeRanges", typeof(List<QualitativeRange>), typeof(RadialGaugeControl), new PropertyMetadata(new List<QualitativeRange>()));

        public List<QualitativeRange> QualitativeRanges
        {
            get { return (List<QualitativeRange>)GetValue(QualitativeRangesProperty); }
            set { SetValue(QualitativeRangesProperty, value); }
        }

        public static readonly DependencyProperty RangesProperty =
            DependencyProperty.Register("Ranges", typeof(IEnumerable<ArcDescriptor>), typeof(RadialGaugeControl), new PropertyMetadata(null));

        public IEnumerable<ArcDescriptor> Ranges
        {
            get { return (IEnumerable<ArcDescriptor>)GetValue(RangesProperty); }
            set { SetValue(RangesProperty, value); }
        }

        public static readonly DependencyProperty MinorTicksProperty =
            DependencyProperty.Register("MinorTicks", typeof(IEnumerable<Tick>), typeof(RadialGaugeControl), new PropertyMetadata(null));

        public IEnumerable<Tick> MinorTicks
        {
            get { return (IEnumerable<Tick>)GetValue(MinorTicksProperty); }
            set { SetValue(MinorTicksProperty, value); }
        }

        public static readonly DependencyProperty MajorTicksProperty =
            DependencyProperty.Register("MajorTicks", typeof(IEnumerable<Tick>), typeof(RadialGaugeControl), new PropertyMetadata(null));

        public IEnumerable<Tick> MajorTicks
        {
            get { return (IEnumerable<Tick>)GetValue(MajorTicksProperty); }
            set { SetValue(MajorTicksProperty, value); }
        }

        public static readonly DependencyProperty ValueAngleProperty =
            DependencyProperty.Register("ValueAngle", typeof(double), typeof(RadialGaugeControl), new PropertyMetadata(null));

        public double ValueAngle
        {
            get { return (double)GetValue(ValueAngleProperty); }
            set { SetValue(ValueAngleProperty, value); }
        }

        #endregion // Dependency Properties

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string prop)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }

        public RadialGaugeControl()
        {
            this.DefaultStyleKey = typeof(RadialGaugeControl);
            this.Ranges = this.getRanges();
            this.MinorTicks = this.getMinorTicks();
            this.MajorTicks = this.getMajorTicks();

            // The following line is necessary, otherwise all ranges in the page wil accumulate ...
            this.QualitativeRanges = new List<QualitativeRange>();
        }

        public Path Needle { get; set; }

        protected override void OnApplyTemplate()
        {
            this.ValueAngle = ValueToAngle(this.Value);
            this.Needle = this.GetTemplateChild(NeedlePartName) as Path;
            this.Needle.RenderTransform = new RotateTransform() { Angle = this.ValueAngle };
            base.OnApplyTemplate();
        }

        private IEnumerable<ArcDescriptor> getRanges()
        {
            for (int i = 0; i < this.QualitativeRanges.Count; i++)
            {
                var range = this.QualitativeRanges[i];
                if (i == 0)
                {
                    yield return new ArcDescriptor()
                    {
                        Color = range.Color,
                        StartAngle = ValueToAngle(this.Minimum),
                        EndAngle = ValueToAngle(range.Maximum)
                    };
                }
                else
                {
                    var previousRange = this.QualitativeRanges[i - 1];
                    yield return new ArcDescriptor()
                    {
                        Color = range.Color,
                        StartAngle = ValueToAngle(previousRange.Maximum),
                        EndAngle = ValueToAngle(range.Maximum)
                    };
                }
            }
        }

        private IEnumerable<Tick> getMinorTicks()
        {
            double tickSpacing = (this.Maximum - this.Minimum) / 50;
            for (double tick = this.Minimum; tick <= this.Maximum; tick += tickSpacing)
            {
                yield return new Tick()
                {
                    Angle = ValueToAngle(tick),
                    Value = tick.ToString("N0"),
                    Height = 200
                };
            }
        }

        private IEnumerable<Tick> getMajorTicks()
        {
            double tickSpacing = (this.Maximum - this.Minimum) / 10;
            for (double tick = this.Minimum; tick <= this.Maximum; tick += tickSpacing)
            {
                yield return new Tick()
                {
                    Angle = ValueToAngle(tick),
                    Value = tick.ToString("N0"),
                    Height = 200
                };
            }
        }

        public double ValueToAngle(double value)
        {   double minAngle = -150;
            double maxAngle = 150;
            
            if (value < this.Minimum)
            {
                return minAngle - 10;
            }

            if (value > this.Maximum)
            {
                return maxAngle + 10;
            }

            double angularRange = maxAngle - minAngle;

            return (value - this.Minimum) / (this.Maximum - this.Minimum) * angularRange + minAngle;
        }

        public class Tick
        {
            public double Angle { get; set; }
            public string Value { get; set; }
            public double Height { get; set; }
        }

        public class ArcDescriptor
        {
            public double StartAngle { get; set; }
            public double EndAngle { get; set; }
            public string Color { get; set; }
        }
    }

    /// <summary>
    /// Defines a single qualitative range value.
    /// </summary>
    public class QualitativeRange
    {
        public double Maximum { get; set; }
        public String Color { get; set; }
    }
}
